#
#       HBS REDUCTION SUB ROUTINE
#
#       Flattening (OBJ+FLAT1+FLAT2)      : hbsflat2.cl
#
#       1997/04/30      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/08/01      K.Kawabata  Ver.0.30
#       1998/02/09      K.Kawabata  Ver.0.50  flat type
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#

procedure hbsflat2

begin

  int flag_mode = 1 # flat type  -1:not use 1:normal
  file fn, fnnexte, fnnexto
  string fn_stack[500]
  int n_image

  string hbslist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string listoflist
  int n_list
  file logfn

  string fnobe1, fnobo1
  string fnf2e1, fnf2o1
  string fnobe, fnobo
  file tmp_list, tmp_fn
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int c, i, j
  string fnflat
  string fn_f2, fn_f5
  int flag_uplow  #  0:lower  1: upper  -1:other

  string caldir, rdir

  int theta_n
  real theta
  file fn_nposhwp = "HBS.nposhwp"
  int n_up = -1

  int flag_erase # 0:keep old images, 1:erase old images
  int flag_inquiry = 0 # 1:'flag_overwrite' applys to all image
  int flag_overwrite = 0 # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "hbs.tmp.time" ) == 1 )
    delete( "hbs.tmp.time", ver- )
  time >> hbs.tmp.time
  list = "hbs.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "hbs.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------------" )
  print( " Flat-fielding"
  print( "---------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------------", >> logfn )
  print( " Flat-fielding", >> logfn )
  print( "---------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  fn_f2 = envget( "_fl2hl2" )
  fn_f5 = envget( "_fl2s21" )
  caldir = envget( "_caldir" )

  tmp_list = mktemp( "hbs.tmp.list." )

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//". Please do hbsgroup." )
    print( "Cannot find "//listoflist//". Please do hbsgroup.", >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n" )
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n", >> logfn )
      bye
    }
  }

#----- read number of position angle of half-wave plate

  if( access( fn_nposhwp ) == 1 )
  {
    list = fn_nposhwp
    if( fscan( list, n_up ) != 1 )
    {
      printf( "Warning: cannot read '%s'\n", fn_nposhwp )
      printf( "Warning: cannot read '%s'\n", fn_nposhwp, >> logfn )
    }
  }
  if( n_up < 0 )
  {
    printf( "Input sequencial number of PA of hwp (1-16) >> " )
    printf( "Input sequencial number of PA of hwp (1-16) >> ", >> logfn )
    c = scan( buf )
    while( int( buf ) < 1 || int( buf ) > 16 )
      c = scan( buf )
    printf( "%s\n", buf, >> logfn )
    n_up = int( buf )

    if( access( fn_nposhwp ) == 1 )
      delete( fn_nposhwp, ver- )
    printf( "%2d\n", n_up, > fn_nposhwp )
  }

#----- scanning the list file

  n_list = 0
  list = listoflist

  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL",\
             >> logfn )
      break
    }
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {

#----- check diaphragm

      fnflat = "none"
      rdir = "none"

      if( substr( diaph[i], 1, 3 ) == "2HL" )
      {
        fnflat = fn_f2
	rdir = caldir
      }
      if( substr( diaph[i], 1, 3 ) == "2S2" )
      {
        fnflat = fn_f5
	rdir = caldir
      }

      if( fnflat == "none" ){
        printf( "Unknown Diaph type in %s (%s)\n", hbslist[i], diaph[i] )
        printf( "Unknown Diaph type in %s (%s)\n", hbslist[i], diaph[i],\
                >> logfn )
	next
      }

      c = strlen( diaph[i] )
      if( substr( diaph[i], 4, 4 ) == "L" )
      {
        flag_uplow = 0
        printf( "--- OBJECT LIST : %s  LOWER (%s) ---\n",\
	        hbslist[i], fnflat )
        printf( "--- OBJECT LIST : %s  LOWER (%s) ---\n",\
	        hbslist[i], fnflat, >> logfn )
      }
      else
      {
        if( substr( diaph[i], 4, 4 ) == "U" )
        {
          flag_uplow = 1
          printf( "--- OBJECT LIST : %s  UPPER (%s) ---\n",\
	          hbslist[i], fnflat )
          printf( "--- OBJECT LIST : %s  UPPER (%s) ---\n",\
	          hbslist[i], fnflat, >> logfn )
        }
        else
        {
          flag_uplow = -1
          printf( "%s did not used two diaphragm mode\n", hbslist[i] )
          printf( "%s did not used two diaphragm mode\n", hbslist[i],\
                  >> logfn )
	  next
        }
      }

#----- choose OBJ spectra

      sed( 's/fit\.imh//', hbslist[i], > tmp_list )

      list = tmp_list
      n_image = 0
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]
        printf( "%s  ", fn )

        if( flag_uplow == 0 )
        {
          fnobe1 = fn//"sf.3"
          fnobo1 = fn//"sf.4"
          fnobe = fn//"fl.3"
          fnobo = fn//"fl.4"

          if( j < n_image )
	  {
            fnnexte = fn_stack[j+1]//"fl.3"
            fnnexto = fn_stack[j+1]//"fl.4"
	  }
	  else
	  {
            fnnexte = "none"
            fnnexto = "none"
	  }

          if( access( fnobe1//".imh" ) == 0 )
          {
            printf( "%s--(not exist. skip.)  ", fnobe1 )
            printf( "%s--(not exist. skip.)  ", fnobe1, >> logfn )
            next
          }

          imgets( fnobe1, 'POS-HWP' )
          theta = real( imgets.value )
          theta_n = int( theta / 22.499999 )

          fnf2e1 = rdir//fnflat//theta_n//".ft.3"
          fnf2o1 = rdir//fnflat//theta_n//".ft.4"
        }
        else
        {
          fnobe1 = fn//"sf.1"
          fnobo1 = fn//"sf.2"
          fnobe = fn//"fl.1"
          fnobo = fn//"fl.2"

          if( j < n_image )
	  {
            fnnexte = fn_stack[j+1]//"fl.1"
            fnnexto = fn_stack[j+1]//"fl.2"
	  }
	  else
	  {
            fnnexte = "none"
            fnnexto = "none"
	  }

          if( access( fnobe1//".imh" ) == 0 )
          {
            printf( "%s--(not exist. skip.)  ", fnobe1 )
            printf( "%s--(not exist. skip.)  ", fnobe1, >> logfn )
            next
          }

          imgets( fnobe1, 'POS-HWP' )
          theta = real( imgets.value )
          theta_n = int( theta / 22.499999 )

          fnf2e1 = rdir//fnflat//theta_n//".ft.1"
          fnf2o1 = rdir//fnflat//theta_n//".ft.2"
        }

	if( access( fnf2e1//".imh" ) == 0 )
	{
	  printf( "FLAT: %s not found. Skip.\n", fnf2e1 )
	  printf( "FLAT: %s not found. Skip.\n", fnf2e1, >> logfn )
	  next
	}
	if( access( fnf2o1//".imh" ) == 0 )
	{
	  printf( "FLAT: %s not found. Skip.\n", fnf2o1 )
	  printf( "FLAT: %s not found. Skip.\n", fnf2o1, >> logfn )
	  next
	}

        if( access( fnobe1//".imh" ) == 0 || access( fnobo1//".imh" ) == 0 )
        {
          printf( "%s(%d)--(not exist. skip.)  ", fn, theta_n )
          printf( "%s(%d)--(not exist. skip.)  ", fn, theta_n, >> logfn )
          next
        }	

        if( access( fnobe//".imh" ) == 1 || access( fnobo//".imh" ) == 1 )
        {
          if( flag_overwrite == 0 )
          {
            if( fnnexte != "none" && ( access( fnnexte//".imh" ) == 0 ||\
	        access( fnnexto//".imh" ) == 0 ) )
            {
	      printf( "\n%s(%d): Maybe insufficient proccess last time.\n",\
	              fn, theta_n )
	      printf( "\n%s(%d): Maybe insufficient proccess last time.\n",\
	              fn, theta_n, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s(%d)--(already exists. skip.)  ", fn, theta_n )
              printf( "%s(%d)--(already exists. skip.)  ",\
                      fn, theta_n, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s(%d) already exist.\n", fn, theta_n )
              printf( "\n%s(%d) already exist.\n", fn, theta_n, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              printf( "%s\n", buf, >> logfn )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s(%d)--(already exists. skip.)  ", fn, theta_n )
                printf( "%s(%d)--(already exists. skip.)  ",\
                        fn, theta_n, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s(%d)--(already exists. skip.)  ", fn, theta_n )
                printf( "%s(%d)--(already exists. skip.)  ",\
                        fn, theta_n, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(%d)(overwrite)  ", fn, theta_n )
          printf( "%s(%d)(overwrite)  ", fn, theta_n, >> logfn )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s(%d)  ", fn, theta_n )
          printf( "%s(%d)  ", fn, theta_n, >> logfn )
        }

#----- flattenning

        if( flag_mode != -1 )
	{
          printf( "%s / %s  --> %s\n", fnobe1, fnf2e1, fnobe )
          printf( "%s / %s  --> %s\n", fnobe1, fnf2e1, fnobe, >> logfn )
          if( access( fnobe//".imh" ) == 1 )
	    imdelete( fnobe, ver- )
          imarith( fnobe1, "/", fnf2e1, fnobe,\
                   pixtype="real", calctype="real", ver- )

          printf( "%s / %s  --> %s\n", fnobo1, fnf2o1, fnobo )
          printf( "%s / %s  --> %s\n", fnobo1, fnf2o1, fnobo, >> logfn )
          if( access( fnobo//".imh" ) == 1 )
	    imdelete( fnobo, ver- )
          imarith( fnobo1, "/", fnf2o1, fnobo,\
                   pixtype="real", calctype="real", ver- )

          if( access( fnobe//".imh" ) == 0 || access( fnobo//".imh" ) == 0 )
          {
            printf( "Flattening failed. Abnormal end.\n" )
            printf( "Flattening failed. Abnormal end.\n", >> logfn )
            bye
          }

          hedit( fnobe, "HISTORY", "Flat-fielding",\
                 delete-, verify-, show-, update+ )
          hedit( fnobo, "HISTORY", "Fat-fielding",\
                 delete-, verify-, show-, update+ )

          if( flag_erase == 1 )
	  {
            imdelete( fnobe1, ver- )
            imdelete( fnobo1, ver- )
	  }
        }
	else
	{
          printf( "%s  --> %s\n", fnobe1, fnobe )
          printf( "%s  --> %s\n", fnobe1, fnobe, >> logfn )
          if( access( fnobe//".imh" ) == 1 )
	    imdelete( fnobe, ver- )
	  if( flag_erase == 1 )
            imrename( fnobe1, fnobe, ver- )
          else
            imcopy( fnobe1, fnobe, ver- )

          printf( "%s  --> %s\n", fnobo1, fnobo )
          printf( "%s  --> %s\n", fnobo1, fnobo, >> logfn )
          if( access( fnobo//".imh" ) == 1 )
	    imdelete( fnobo, ver- )
	  if( flag_erase == 1 )
            imrename( fnobo1, fnobo, ver- )
          else
            imcopy( fnobo1, fnobo, ver- )
        }
      }
      delete( tmp_list, ver- )
    }
  }

end

