#
#       HBS REDUCTION SUB ROUTINE
#
#       Grouping FLAT lists : hbsflatgroup.cl
#
#       1997/04/22      K.Kawabata   ver 1.00
#       1997/09/03      K.Kawabata   ver 1.10  1-hole diaphragm
#       1998/04/20      K.Kawabata   ver 2.00  unification
#

procedure hbsflatgroup

begin

  int i, j, k, n_list, n_flatlist
  file fn_head, fn, fn_tmp
  string listoflist, flatlist, tmp_list
  string outlist[50], outlistsub, outfile[50], outfilesub
  int n_image
  string fl1hl1, fl2hl1, fl2hl2, fl2hl3
  string fl2s21, fl2s22
  string hbslist[100]
  string shorttyp[100], title[100]
  string exptime[100], diaph[100], filter[100]
  string slit[100]
  string listdiaph[30], listslit[30]
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  string buf="y"
  real theta
  int theta_n
  int n_up = -1
  file fn_nposhwp
  
  print( "" )
  print( "---------------------" )
  print( " Grouping flat lists" )
  print( "---------------------" )
  print( "" )

  listoflist = envget( "_rootlist" )
  flatlist = envget( "_flatlist" )
  fl1hl1 = envget( "_fl1hl1" )
  fl2hl1 = envget( "_fl2hl1" )
  fl2hl2 = envget( "_fl2hl2" )
  fl2hl3 = envget( "_fl2hl3" )
  fl2s21 = envget( "_fl2s21" )
  fl2s22 = envget( "_fl2s22" )
  fn_nposhwp = envget( "_nposhwp" )

  tmp_list = mktemp( "hbs.tmp.list." )

#----- Delete old flat list and flat images

  if( access( flatlist ) )
  {
    printf( "There is old flat list(%s).  delete ? (y/n)  >> ", flatlist )
    k = scan( buf )
    while( buf != "y" && buf != "n" )
       k = scan( buf )
    if( buf == "y" )
      delete( flatlist )
    else
    {
      print( "OK. continue refering these files" )
      bye
    }
  }

  sections( "HBS.FLAT.*", option="fullname", > tmp_list )
  if( sections.nimages > 0 )
  {
    print( "There are old flat lists" )
    type( tmp_list )
    printf( "delete them all ?  (y/n)  >> " )
    k = scan( buf )
    while( buf != "y" && buf != "n" )
       k = scan( buf )
    if( buf == "y" )
      delete( "@"//tmp_list )
    else
    {
      print( "Can't continue. Please delete or rename old lists(HBS.FLAT.*)" )
      bye
    }
  }
  delete( tmp_list ) 


#----- read number of position angle of half-wave plate

  if( access( fn_nposhwp ) == 1 )
  {
    list = fn_nposhwp
    if( fscan( list, n_up ) != 1 )
    {
      printf( "Warning: cannot read '%s'\n", fn_nposhwp )
    }
  }
  if( n_up < 0 )
  {
    printf( "Input sequencial number of PA of hwp (1-16) >> " )
    c = scan( buf )
    while( int( buf ) < 1 || int( buf ) > 16 )
      c = scan( buf )
    n_up = int( buf )

    if( access( fn_nposhwp ) == 1 )
      delete( fn_nposhwp, ver- )
    printf( "%2d\n", n_up, > fn_nposhwp )
  }
 
#----- Scanning the list file (made by hbsgroup)

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file "//listoflist//" is not produced by HBSGROUP.CL" )
      break
    }
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- Re-grouping flat images

  print( " Now grouping flat images..." )
  n_flatlist = 0
  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] == "FLA" )
    {
      sed( 's/fit\.imh//', hbslist[i], > tmp_list )
      list = tmp_list
      while( fscan( list, fn_head ) != EOF )
      {
        fn = fn_head//"tr.imh"
        fn_tmp = fn_head//"px.imh"
	if( access( fn ) == 1 )
        {
	  imgets( fn, "POS-HWP" )
	}
	else{
	  if( access( fn_tmp ) == 1 )
          {
	    imgets( fn_tmp, "POS-HWP" )
	  }
	  else
	  {
	    printf( "Not found '%s' nor '%s'. Ignored.\n", fn, fn_tmp )
	    next
	  }
	}
        theta = real( imgets.value )
        theta_n = int( theta / 22.499999 )

	outlistsub = ""
	outfilesub = ""

	if( substr( diaph[i], 1, 3 ) == "1HL" )
	{
	  outlistsub = "HBS."//fl1hl1//theta_n
	  outfilesub = fl1hl1//theta_n//".px"
	}

        if( substr( diaph[i], 1, 3 ) == "2HL" )
        {
          if( real( slit[i] ) < 0.6 )
          {
            outlistsub = "HBS."//fl2hl2//theta_n
            outfilesub = fl2hl2//theta_n//".px"
          }
          else
          {
            if( substr( title[i], 1, 1 ) != "U" )
            {
              outlistsub = "HBS."//fl2hl1//theta_n
              outfilesub = fl2hl1//theta_n//".px"
            }
            else
	    {
              outlistsub = "HBS."//fl2hl3//theta_n
              outfilesub = fl2hl3//theta_n//".px"
	    }
	  }
        }

        if( substr( diaph[i], 1, 3 ) == "2S2" )
        {
          outlistsub = "HBS."//fl2s21//theta_n
          outfilesub = fl2s21//theta_n//".px"
        }

	if( outlistsub == "" || outfilesub == "" )
	  next
        if( n_flatlist == 0 )
          goto nextstep1
        else
        {
          k = 0
          for( j=1; j<=n_flatlist; j+=1 )
          {
            if( outlistsub == outlist[j] )
            k = 1
          }
          if( k == 0 )
            goto nextstep1
        }
        goto nextstep2

nextstep1:
        n_flatlist += 1
        outlist[n_flatlist] = outlistsub
        outfile[n_flatlist] = outfilesub
	listdiaph[n_flatlist] = diaph[i]
	listslit[n_flatlist] = slit[i]

nextstep2:
        print( fn, >> outlistsub )
      }
      delete( tmp_list ) 
    }
  }

#----- make flatlist

  #---- (1HOLE)
  i=1
  for( j=0; j<=(n_up-1); j+=1 )
  {
    for( k=1; k<=n_flatlist; k+=1 )
    {
      if( outlist[k] == "HBS."//fl1hl1//j )
      {
        printf( "%12s %12s %1d %1d %5s %s\n", outlist[k], outfile[k],\
         i, j, listdiaph[k], listslit[k],  >> flatlist )
      }
    }
  }
  #---- (2HOLE_flat1)
  i=2
  for( j=0; j<=(n_up-1); j+=1 )
  {
    for( k=1; k<=n_flatlist; k+=1 )
    {
      if( outlist[k] == "HBS."//fl2hl1//j )
      {
        printf( "%12s %12s %1d %1d %5s %s\n", outlist[k], outfile[k],\
         i, j, listdiaph[k], listslit[k],  >> flatlist )
      }
    }
  }
  #---- (2HOLE_flat2)
  i=3
  for( j=0; j<=(n_up-1); j+=1 )
  {
    for( k=1; k<=n_flatlist; k+=1 )
    {
      if( outlist[k] == "HBS."//fl2hl2//j )
      {
        printf( "%12s %12s %1d %1d %5s %s\n", outlist[k], outfile[k],\
         i, j, listdiaph[k], listslit[k],  >> flatlist )
      }
    }
  }
  #---- (2HOLE_flat2_UP)
  i=4
  for( j=0; j<=(n_up-1); j+=1 )
  {
    for( k=1; k<=n_flatlist; k+=1 )
    {
      if( outlist[k] == "HBS."//fl2hl3//j )
      {
        printf( "%12s %12s %1d %1d %5s %s\n", outlist[k], outfile[k],\
         i, j, listdiaph[k], listslit[k],  >> flatlist )
      }
    }
  }
  #---- (2SLIT02)
  i=5
  for( j=0; j<=(n_up-1); j+=1 )
  {
    for( k=1; k<=n_flatlist; k+=1 )
    {
      if( outlist[k] == "HBS."//fl2s21//j )
      {
        printf( "%12s %12s %1d %1d %5s %s\n", outlist[k], outfile[k],\
         i, j, listdiaph[k], listslit[k],  >> flatlist )
      }
    }
  }

#----- confirmation

  printf( "\n********** grouping done ***********\n\n" )
  for( i=1; i<=n_flatlist; i+=1 )
  {
    printf( "---- No.%d  %s ----\n", i, outlist[i] )
    type( outlist[i] )
    printf( "\n" )
  }

#nextstep10:
#  buf = "y"
#  printf( "OK ?  [y]:yes [n]:no >> " )
#  k = scan( buf )
#  while( buf != "y" && buf != "n" )
#    k = scan( buf )
#  if( buf == "n" )
#  {
#    print( "Please correct the list files (FLAT.*) in an other terminal" )
#    goto nextstep10
#  }

end
