#
#       HBS REDUCTION SUB ROUTINE
#
#      Prescan and make template four spectra (2HOLES)
#
#                                             : hbsprescan1.cl
#
#
#
#       1997/03/21      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/04/10      K.Kawabata  Ver.0.20  refer HBS.list
#


procedure hbsprescan


begin

  file listoflist = "HBS.list"
  file fn_scan = "prescan1.px"
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  file tmp_fn1, tmp_list
  int c, i, j, n_list
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  real slicelevel = 0.005
  real dy1 = -15.0
  real dy2 = 15.0

  tmp_fn = mktemp( "hbs.tmp.image." )
  tmp_list = mktemp( "hbs.tmp.list." )

  print( "" )
  print( "--------------------------------------------------------------" )
  print( " make a template of distortion scanning four spectra (2HOLES)" )
  print( "--------------------------------------------------------------" )
  print( "" )


#---- confirmation

  if( access( fn_scan//".imh" ) )
  {
    print( "Detected old '"//fn_scan//"'. Use this ? (y/n)  >> " )
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      goto prescanning
    imdelete( fn_scan, ver- )
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }
  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 != "OBJ" || buf3 == "SKY" || buf3 == "sky" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#---- selection of images for average

  for( i=1; i<=n_list; i+=1 )
  {
    printf( "%2d: %16s  %6s  %s\n", i, hbslist[i], title[i], diaph[i] )
  }
nextstep1:
  printf( "Choose an UPPER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep1
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "UPPER" )
  {
    printf( "%s is not using UPPER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep1
  }
  sed( 's/\.fit\.imh/\.px\.imh/', hbslist[i], > tmp_list )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep1
  
nextstep2:
  printf( "Choose a LOWER-HOLE group  >> " )
  while( scan( buf ) != 1 ){}
  i = int( buf )
  if( i < 1 || i > n_list )
    goto nextstep2
  c = strlen( diaph[i] )
  if( substr( diaph[i], (c-4), c ) != "LOWER" )
  if( diaph[i] != "2HOLES_LOWER" )
  {
    printf( "%s is not using LOWER HOLE !!  avoided.\n", hbslist[i] )
    goto nextstep2
  }
  sed( 's/\.fit\.imh/\.px\.imh/', hbslist[i], >> tmp_list )
  printf( "OK. %s is registed.  More ? [y/n]  >> ", hbslist[i] )
  i = scan( buf )
  while( buf != "y" && buf != "n" )
    i = scan( buf )
  if( buf == "y" )
    goto nextstep2

#---- make reference image

  imcombine( "@"//tmp_list, fn_scan, combine="average",\
                 reject="none", project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="none", lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1 )
  delete( tmp_list, ver- )

#---- scanning four spectra

prescanning:

  if (access('database/ap'//fn_scan))
    delete('database/ap'//fn_scan)

#  display( fn, 1, fill+ )
  hedit( fn, "DISPAXIS", "1", add+, delete-, verify-, show-, update+ )

  apall( input=fn_scan, nfind=4, output=tmp_fn, format="strip",\
         reference="", profiles="", interac+, find+, recenter+, resize+,\
	 edit+, trace+, fittrace+, extract-, extras-, review-, line=INDEF,\
	 nsum=3, lower=dy1, upper=dy2, apidtable="", width=30.,\
	 radius=30., threshold=0., minsep=30., maxsep=1000.,\
	 order="increasing", apertures="", npeaks=INDEF, shift-,\
	 llimit=dy1, ulimit=dy2, ylevel=slicelevel, peak+, bkg-,\
	 r_grow=0., avglimits-, t_nsum=3, t_step=3, t_nlost=10,\
	 t_function="legendre", t_order=6, t_sample="*", t_niterate=0,\
	 t_low_reject=3., t_high_reject=3., t_grow=0., background="none",\
	 weights="none", pfit="fit1d", clean-, skybox=1, saturation=50000,\
	 readnoise=8., gain=0.26, lsigma=4., usigma=4. )

  buf1 = fn_scan//".1"
  buf2 = fn_scan//".2"
  buf3 = fn_scan//".3"
  buf4 = fn_scan//".4"
  hbsapext( fn_scan, fn_scan, buf1, buf2, buf3, buf4 )

end
