#
#       HBS REDUCTION SUB ROUTINE
#
#       sensitivity correction using UP spectra : hbssenscor.cl
#
#       1998/03/08      K.Kawabata  Ver.1.00  use 'HBS.list'
#

procedure hbssenscor

begin

  int flag_unpolflat = 0  # 0: normal flat(use upave), 1: unpolarized flat
  string rdir

  file listoflist
  string hbslist[50]
  string shorttyp[50], title[50]
  string exptime[50], diaph[50], filter[50]
  string slit[50]
  int c, i, j, n, n_list
  string fn_stack[300]
  int n_image

  file fn, fnnexte, fnnexto
  string fnobe, fnobo, fnupe, fnupo
  file tmp_list, tmp_fn
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7

  string sens_2hl, sens_2s2
  string sens_fn

  int flag_erase # 0:keep old images, 1:erase old images
  int flag_inquiry   # 1: 'flag_overwrite' applys to all image
  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display

#--- initial settings

  listoflist = envget( "_rootlist" )
  rdir = envget( "_caldir" )
  sens_2hl = rdir//envget( "_fl2hls" )
  sens_2s2 = rdir//envget( "_fl2s2s" )

  print( "" )
  print( "-----------------------" )
  print( " sensitivity correcton"
  print( "-----------------------" )
  print( "" )

  tmp_list = mktemp( "hbs.tmp.list." )
  tmp_fn = mktemp( "hbs.tmp.image." )

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_ver) Retry.\n" )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'hbsinit' process is incomplete.(_fg_mode) Retry.\n" )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'hbsinit' process is incomplete.(_fg_del2) Retry.\n" )
      bye
    }
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do hbsgroup" )
    bye
  }

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "HBS" )
    {
      print( "The list-file ", listoflist, " is not produced by HBSGROUP.CL" )
      break
    }
    if( buf2 != "OBJ" && buf2 != "COM" )
      next
    n_list += 1
    hbslist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine ----

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" )
    {
        sens_fn = "none"

#----- diaphragm

      if( substr( diaph[i], 1, 3 ) == "2HL" )
      {
        sens_fn = sens_2hl
      }
      if( substr( diaph[i], 1, 3 ) == "2S2" )
      {
        sens_fn = sens_2s2
      }

      if( sens_fn == "none" ){
        printf( "Unknown Diaph type in %s (%s)\n", hbslist[i], diaph[i] )
	next
      }

      printf( "--- OBJECT LIST : %s ---\n", hbslist[i] )
      sed( 's/fit\.imh//', hbslist[i], > tmp_list )

#----- read indivisual list file

      list = tmp_list
      n_image = 0
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]

        fnobe = fn//"co.e"
        fnobo = fn//"co.o"

        buf1 = fn//"se.e"
        buf2 = fn//"se.o"

	if( j < n_image )
        {	  
          fnnexte = fn_stack[j+1]//"se.e"
          fnnexto = fn_stack[j+1]//"se.o"
        }
	else
	{
          fnnexte = "none"
          fnnexto = "none"
	}

	if( access( fnobe//".imh" ) == 0 || access( fnobo//".imh" ) == 0 )
	{
          printf( "%s--(not exist. skip.)  ", fn )
          next
	}


        if( access( fnobe//".imh" ) == 1 || access( fnobo//".imh" ) == 1 )
        {
          if( flag_overwrite == 0 )
          {
            if( fnnexte != "none" && ( access( fnnexte//".imh" ) == 0 ||\
	        access( fnnexto//".imh" ) == 0 ) )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn )
        }
        else
          if( flag_ver == 1 )
            printf( "%s  ", fn )

#----- divide by UP spectrum

        if( access( buf1//".imh" ) == 1 )
	  imdelete( buf1, ver- )
        if( access( buf2//".imh" ) == 1 )
	  imdelete( buf2, ver- )

        if( flag_unpolflat == 0 ){
          imarith( fnobe, "/", sens_fn, buf1,\
                 pixtype="real", calctype="real", ver- )
          imarith( fnobo, "/", sens_fn, buf2,\
                 pixtype="real", calctype="real", ver- )
        }
	else
	{
          imcopy( fnobe, buf1, ver- )
          imcopy( fnobo, buf2, ver- )
	}

      }
      delete( tmp_list, ver- )
    }
  }


end
