/***********************************************************

    hbs reduction subroutine: hbs1dsub.c

                                   1997/07/08  K. Kawabata

    usage: % hbs1dsub infile specno fn_postfit outfile tmpfile

      infile -- input image file (FITS format)

      specno -- spectrum number

      fn_postfit -- aperture data made by 'hbspostfit'

      outfile -- output spectrum file (FITS format)

      tmpfile -- temporary file (y_peak written)

************************************************************/
		  
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include"fitsio.h"

#include"hbsdefine.h"

int readpostfit( char *infile, double coef[][6]  ){

  FILE *fp;
  int i;
  char buf[100];

  if( ( fp = fopen( infile, "rt" ) ) == NULL ){
    return( -1 );
  }

  for( i=1; i<=8; i++ ){
    fgets( buf, 100, fp );
    sscanf( buf, "%lf %lf %lf %lf %lf %lf", &coef[i][0], &coef[i][1],\
	    &coef[i][2], &coef[i][3], &coef[i][4], &coef[i][5] );
  }

  fclose( fp );
  return( 0 );

}

int scanfile( char *infile, float count[][NX2]  ){

  fitsfile *fp;
  int i, anynul, status;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, infile, READONLY, &status ) != 0 ){
    printf( "cannot open file '%s' status=%d\n", infile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 || naxis2 != NY2 ){
    printf( "error: definition of NX2 or NY2 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_read_img( fp, TFLOAT, 1L, npix, &nulval,\
      count[0], &anynul, &status ) != 0 ){
    printf( "cannot read from '%s'\n", infile );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }
  return( 0 );
}


double realcount( float c[][NX2], int x, double y_low, double y_high ){

  int iy_low, iy_high;
  int i, j;
  double count = 0.0;


  if( x < 1 || x > NX2 || y_low >= y_high || ( y_low < 0.5 && y_high < 0.5 ) || ( y_low > ( NY2 + 0.5 ) && y_high > ( NY2 + 0.5 ) ) ){
    count = 0.0;
  }
  else{
    if( y_low < 0.5 )
      y_low = 0.5;
    if( y_high > ( NY2 + 0.5 ) )
      y_high = NY2 + 0.5;

    iy_low = y_low + 1.4999999999;
    iy_high = y_high - 0.4999999999;

    if( iy_low <= iy_high ){
      for( j=iy_low; j<=iy_high; j++ ){
        count += c[j-1][x-1];
      }
    }
    if( ( iy_high - iy_low ) > -2 ){
      if( (iy_low - 2) >= 0 )
	count += ( ( ( iy_low - 0.5 ) - y_low ) * c[iy_low-2][x-1] );
      if( iy_high <= ( NY2 - 1 ) )
	count += ( ( y_high - ( iy_high + 0.5 ) ) * c[iy_high][x-1] );
    }
    else{
      iy_high = iy_high + 1;
      iy_low = iy_low - 1;
      if( iy_high != iy_low ){
	printf( "Error in realcount\n" );
	return( 0.0 );
      }
      count += ( ( y_high - y_low ) * c[iy_high-1][x-1] );
    }
  }
  return( count );
}

double findpeak( float c[][NX2], int x, double ref_pos, double wid ){

  double y_low, y_high;
  int iy_low, iy_high;
  int i, j;
  double count;
  double ycen;
  int nloop = 10;
  double tmp;

  ycen = ref_pos;

  for( i=1; i<=nloop; i++ ){

    count = 0.0;

    y_low  = ycen - wid / 2.0;
    y_high = ycen + wid / 2.0;

    if( x < 1 || x > NX2 || y_low >= y_high || ( y_low < 0.5 && y_high < 0.5 ) || ( y_low > ( NY2 + 0.5 ) && y_high > ( NY2 + 0.5 ) ) ){
      count = 0.0;
    }
    else{
      if( y_low < 0.5 )
	y_low = 0.5;
      if( y_high > ( NY2 + 0.5 ) )
	y_high = NY2 + 0.5;

	iy_low = y_low + 1.4999999999;
	iy_high = y_high - 0.4999999999;

      if( iy_low <= iy_high ){
	for( j=iy_low; j<=iy_high; j++ ){
	  count += ( c[j-1][x-1] * ( j - ycen ) );
	}
      }

      if( ( iy_high - iy_low ) > -2 ){
	if( (iy_low - 2) >= 0 )
	  count += ( ( ( iy_low - 0.5 ) - y_low ) * c[iy_low-2][x-1] ) *\
	    ( iy_low - 1.0 - ycen );
	if( iy_high <= ( NY2 - 1 ) )
	  count += ( ( y_high - ( iy_high + 0.5 ) ) * c[iy_high][x-1] ) *\
	    ( iy_high + 1.0 - ycen );
      }
      else{
	iy_high = iy_high + 1;
	iy_low = iy_low - 1;
	if( iy_high != iy_low ){
	  printf( "Error in findpeak\n" );
	  return( 0.0 );
	}
	count += ( ( y_high - y_low ) * c[iy_high-1][x-1] ) * ( iy_high - ycen );
      }
    }
    tmp = realcount( c, x, y_low, y_high );
    if( fabs( tmp ) > 1.0e-6 )
      ycen += ( count / tmp );
  }
  return( ycen );
}

double findmeanpeak( float count1[][NX2], double f_width ){

  float count4[NY2][NX2];
  double ypeak, ypeak0;
  int x;
  int i, j;

  ypeak0 = ( NY2 - 1.0 ) / 2.0;
  ypeak = 0.0;

  for( i=1; i<=NX2; i++ )
    for( j=1; j<=NY2; j++ )
      count4[j-1][i-1] = 0.0;

  for( i=1; i<=NX2; i++ )
    if( i > (NX2*0.3) && i < (NX2*0.9 ) )
      for( j=1; j<=NY2; j++ )
	count4[j-1][0] += count1[j-1][i-1];
/*	count4[j-1][0] += count1[j-1][i-1]; */

  ypeak = findpeak( count4, 1, ypeak0, f_width );

  return( ypeak );
/*  return( ypeak0 ); */

}

double sp1d( float count1[][NX2], double coefl[], double coefh[],\
	  float count2[] ){

  int x, anynul, status;
  double ypeak, ylthr, yhthr, dythr;
  char buf[100];
  float nulval = 0.;
  long naxis1, naxis2, npix;
  char comment[100];
  double f_width;
  double dylim;

  f_width = NY2a;
  dylim = NY2a;

  ypeak = findmeanpeak( count1, f_width );

  if( ypeak < dylim / 2.0 ){
    ypeak = dylim / 2.0;
  }
  else{
    if( ypeak > ( NX2 - dylim / 2.0 ) ){
      ypeak = NX2 - dylim / 2.0;
    }
  }

  for( x=1; x<=NX2; x++ ){
/*
    ylthr = coefl[0] + coefl[1]*x + coefl[2]*x*x + coefl[3]*x*x*x +\
	    coefl[4]*pow( x, 4.0 ) + coefl[5]*pow( x, 5.0 );
    yhthr = coefh[0] + coefh[1]*x + coefh[2]*x*x + coefh[3]*x*x*x +\
	    coefh[4]*pow( x, 4.0 ) + coefh[5]*pow( x, 5.0 );
    dythr = yhthr - ylthr;
    if( dythr > dylim ){
        printf( "dythr (%5.2f) exceed dtlim. (%5.2f) Neglected.\n", dythr, dylim );
	dythr = dylim;
    }
*/

    dythr = dylim;

/*
    dythr = 21.;
*/

/*
    if( x == NX2 / 2 )
      printf( "x %3d  dy=%5.2f  ", x, dythr );
*/
    count2[x-1] = realcount( count1, x, (ypeak-dythr/2.0), (ypeak+dythr/2.0) );
  }
  return( ypeak );
}

int writefile( char *outfile, float count[] ){

  fitsfile *fp;
  int i, status;
  char buf[100];
  long naxis1, naxis2, npix;
  char comment[100];

  status = 0;

  if( fits_open_file( &fp, outfile, READWRITE, &status ) != 0 ){
    printf( "Cannot open file '%s' with READWRITE mode. status=%d\n",\
	    outfile, status );
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS1", &naxis1,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( fits_read_key( fp, TLONG, "NAXIS2", &naxis2,\
      comment, &status ) != 0 ){
    return( status );
  }
  if( naxis1 != NX2 ){
    printf( "Error: definition of NX2 in 'hbsdefine.h'\n" );
    return( -1 );
  }
  if( naxis2 != 1 ){
    printf( "Error: definition of y-dimension in 'hbsdefine.h'\n" );
    return( -1 );
  }
  npix = naxis1 * naxis2;

  if( fits_write_img( fp, TFLOAT, 1, npix,\
      count, &status ) != 0 ){
    printf( "Error in the function fits_write_img\n" );
    return( status );
  }
  if( fits_close_file( fp, &status ) != 0 ){
    return( status );
  }

  return( 0 );
}

int main( int argc, char *argv[] ){

  double coef[9][6];
  float count1[NY2][NX2];
  float count2[NX2];
  int i, specno;
  char infile[50], outfile[50], fn_postfit[50];
  double y_peak;
  FILE *fp;
  char tmpfile[50];

  double weightr = 0.5;
  double cutoffr = 0.5;

  if( argc != 6 ){
    printf( "usage: hbs1dsub infile specno fn_postfit outfile tmpfile\n" );
    exit( -1 );
  }

  sprintf( infile,"%s", argv[1] );
  specno = atoi( argv[2] );
  strcpy( fn_postfit, argv[3] );
  strcpy( outfile, argv[4] );
  strcpy( tmpfile, argv[5] );

  if( strcmp( fn_postfit, "none" ) != 0 && strcmp( fn_postfit, "NONE" ) != 0 ){    while( readpostfit( fn_postfit, coef ) != 0 ){
      printf( "Read error in '%s'\n", fn_postfit );
      printf( "Do 'hbspostfit' again !!\n" );
      return( 1 );
    }
  }

  if( scanfile( infile, count1 ) != 0 ){
    printf( "read error in '%s'\n", infile );
    return( 1 );
  }

  y_peak = sp1d( count1, coef[specno*2-1], coef[specno*2], count2 );

  fp = fopen( tmpfile, "wt" );
  fprintf( fp, "%5.2f\n", y_peak );
  fclose( fp );

  if( writefile( outfile, count2 ) != 0 ){
    printf( "write failed in '%s'\n", outfile );
    return( 1 );
  }

  return( 0 );

}
